﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Persistence {
  [NonDiscoverableType]
  internal sealed class ShortIndex<T> where T : class {
    private Dictionary<T, ushort> indexes;
    private Dictionary<ushort, T> values;
    private ushort nextIndex;

    public ShortIndex() {
      indexes = new Dictionary<T, ushort>();
      values = new Dictionary<ushort, T>();
      nextIndex = 1;
    }
    public ShortIndex(IEnumerable<Tuple<ushort, T>> values)
      : this() {
      foreach (var value in values) {
        this.indexes.Add(value.Item2, value.Item1);
        this.values.Add(value.Item1, value.Item2);
        nextIndex++;
      }
    }

    public ushort GetIndex(T value) {
      ushort index = 0;
      indexes.TryGetValue(value, out index);
      if (index == 0) {
        index = nextIndex;
        nextIndex++;
        indexes.Add(value, index);
        values.Add(index, value);
      }
      return index;
    }
    public T GetValue(ushort index) {
      return values[index];
    }
    public IEnumerable<Tuple<ushort, T>> GetValues() {
      return values.Select(x => new Tuple<ushort, T>(x.Key, x.Value));
    }
  }
}
